import { Component, Input, OnInit, Inject, NgZone, PLATFORM_ID  } from '@angular/core';
import { isPlatformBrowser } from '@angular/common';
import { Chart } from 'angular-highcharts';

// amCharts imports
import * as am4core from '@amcharts/amcharts4/core';
import * as am4charts from '@amcharts/amcharts4/charts';
import am4themes_animated from '@amcharts/amcharts4/themes/animated';

@Component({
  selector: 'byte-sankey-diagram',
  templateUrl: './sankey-diagram.component.html',
  styleUrls: ['./sankey-diagram.component.scss']
})
export class SankeyDiagramComponent implements OnInit {

  chart: am4charts.XYChart;
  startPoint = 'Session start';
  private percentDetail = {};

  @Input()
  identificador;

  @Input()
  data;

  prefix_intent = 'intencion_';

  constructor(@Inject(PLATFORM_ID) private platformId, private zone: NgZone) {}


  // Run the function only in the browser
  browserOnly(f: () => void) {
    if (isPlatformBrowser(this.platformId)) {
      this.zone.runOutsideAngular(() => {
        f();
      });
    }
  }

  ngAfterViewInit() {
    
    // Chart code goes in here
    this.browserOnly(() => {
      am4core.useTheme(am4themes_animated);
      am4core.addLicense("ch-custom-attribution");

      let chart = am4core.create(this.identificador, am4charts.SankeyDiagram);

      chart.data = this.generateData(this.data);
      chart.nodes.template.nameLabel.label.truncate = false;
      chart.nodes.template.nameLabel.label.wrap = true;

      // espacio entre nodos
      chart.nodePadding = 50;

      chart.dataFields.fromName = "from";
      chart.dataFields.toName = "to_short";
      chart.dataFields.value = "value";
      chart.paddingRight = 100;
      chart.paddingLeft = 30;
      chart.paddingTop    = 10;
      chart.paddingBottom = 10;


      var nodeTemplate = chart.nodes.template;
      nodeTemplate.width = 100;
      // texto dentro de la caja
      nodeTemplate.nameLabel.locationX = 0.2;
      // nodeTemplate.minHeight
      nodeTemplate.showTooltipOn  = "hover";
      nodeTemplate.tooltipText =  `
      [bold]{to}[/]   \n \n
      Número de mensajes
      {value} \n \n
      Porcentaje del total de mensajes 
      {percent} %[/]
      `;
      nodeTemplate.cursorOverStyle = am4core.MouseCursorStyle.pointer;
      nodeTemplate.stroke = am4core.color("#fff");
      nodeTemplate.strokeWidth = 2;
      nodeTemplate.nameLabel.locationX = 0.1;
      nodeTemplate.nameLabel.label.fill = am4core.color("#fff");
      nodeTemplate.nameLabel.label.fontWeight = "bold";
      nodeTemplate.fontSize = 12;
      
      nodeTemplate.nameLabel.label.width = 150;
      nodeTemplate.width = 100;
      nodeTemplate.minHeight  = 50;
      nodeTemplate.maxHeight  = 50;
      // nodeTemplate.measuredHeight = 10;
      nodeTemplate.height = 10;
      nodeTemplate.paddingBottom = 10;
      nodeTemplate.marginBottom = 300;
      nodeTemplate.margin(0, 0, 0, 0);


      var linkTemplate = chart.links.template;
      // linkTemplate.tooltipText = `
      //   {fromName} → {toName}: [bold]{value}[/] \n \n
      //   [bold]Ayuda
      //   (Contenido del mensaje) \n \n
      //   %[/] de todos los mensajes 
      //   {percent} %[/]\n \n
      //   Número de mensajes
      //   {value}
      // `;
      linkTemplate.minHeight = 30;
      linkTemplate.tension = 0.6;
      var hoverState = linkTemplate.states.create("hover");
      hoverState.properties.fillOpacity = 1;
      linkTemplate.controlPointDistance = 0.1;
      linkTemplate.fill = am4core.color("#A8C686");
      
    });
  }

  ngOnDestroy() {
    // Clean up chart when the component is removed
    this.browserOnly(() => {
      if (this.chart) {
        this.chart.dispose();
      }
    });
  }

  ngOnInit() {
    console.log(' id sankey :',this.identificador,this.data);
    if (!this.data) {
      return;
    }
    // console.log('Sankey data ',this.data);
    const dataDiagram = this.generateData(this.data);
    if (!dataDiagram) {
      return;
    }
    
  }

  private generateData(data: any) {
    if (!data['topList']) {
      return undefined;
    }

    const dataForDiagram = [];
    data['topList'].forEach(x => {
      // PercentageInventary.getInstance().setPercent(x['identifier'], x['count']/data['total']*100);
      dataForDiagram.push({
        from:this.startPoint, 
        to: x['identifier'], 
        value: x['count']});
    });
    let total_cont = 0;
    dataForDiagram.forEach(x => {
      total_cont = total_cont + x['value'];
    });
    const columnsData = [];
    dataForDiagram.forEach(x => {
      
      let long_label = x['to'];
      
      if (x['to'].includes(this.prefix_intent)){
        long_label = x['to'].split(this.prefix_intent)[1];
      }
      if (x['to'].includes('intenccion_')){
        long_label = x['to'].split('intenccion_')[1];
      }
      let short_label = long_label.substring(0,6);
      if (x['to'].length > 6){
        short_label = short_label +'...';
      }
      columnsData.push({
        from: x['from'],
        to: long_label,
        to_short: short_label,
        value: x['value'],
        percent: x['value']/total_cont*100
      });
    });
    // columnsData[0]['to_short'] = this.startPoint;
    console.log(columnsData);

    return columnsData;
  }

}

function update(key) {
  return PercentageInventary.getInstance().getPercent(key);
}

export class PercentageInventary {

  private static instance: PercentageInventary;
  private percentages: any = {};

  public static getInstance(): PercentageInventary {
    if (!PercentageInventary.instance) {
      PercentageInventary.instance = new PercentageInventary();
    }

    return PercentageInventary.instance;
  }

  getPercent(name: string) {
    if (this.percentages[name]) {
      return (this.percentages[name] as number).toFixed(2) + '%';
    }
    return null;
  }

  setPercent(name: string, value: any) {
    this.percentages[name] = value;
  }
}
