import { DecimalPipe } from '@angular/common';
import { Component, Input, OnInit } from '@angular/core';

/* Imports */
import * as am4core from "@amcharts/amcharts4/core";
import * as am4charts from "@amcharts/amcharts4/charts";
import am4themes_animated from "@amcharts/amcharts4/themes/animated";


export class GaugeChart {
  hasNeedle?: boolean;
  needleColor?: string;
  needleUpdateSpeed?: number;
  arcColors?: Array<string>;
  arcDelimiters?: Array<number>;
  rangeLabel?: Array<string>;
  needleStartValue?: number;
}

export const DEFAULT_OPTIONS_GAUGE_CHART: GaugeChart = {
  hasNeedle: true,
  needleColor: '#676a6c',
  needleUpdateSpeed: 1000,
  arcColors: ['#1ab394', '#f8ac59', '#ed5565'],
  arcDelimiters: [16, 33],
  rangeLabel: ['0', '6'],
  needleStartValue: 1,
};

@Component({
  selector: 'byte-gauge-chart',
  templateUrl: './gauge-chart.component.html',
  styleUrls: ['./gauge-chart.component.scss']
})
export class GaugeChartComponent implements OnInit {

  @Input()
  name = 'Gauge chart';

  @Input()
  value: any = 1.2;

  @Input()
  unit = '';

  

  @Input()
  options: GaugeChart;

  @Input()
  format = false;

  @Input()
  maxValue = 6;

  canvasWidth = 300;
  centralLabel = '25';
  bottomLabel = '';

  constructor() { }

  ngOnInit() {

    ////inicio
    am4core.useTheme(am4themes_animated);
    am4core.addLicense("ch-custom-attribution");
// Themes end

// create chart
let chart = am4core.create("chartdiv-gauge", am4charts.GaugeChart);
chart.hiddenState.properties.opacity = 0; // this makes initial fade in effect

chart.innerRadius = am4core.percent(60);

chart.radius= am4core.percent(90);

let axis = chart.xAxes.push( new am4charts.ValueAxis() as any);
axis.min = 0;
axis.max = 30;
axis.strictMinMax = true;
const valor = parseInt(this.value, 10);

axis.renderer.grid.template.stroke = new am4core.InterfaceColorSet().getFor("background");
axis.renderer.grid.template.strokeOpacity = 0.3;
axis.renderer.labels.template.fontSize = 10;
axis.renderer.labels.template.adapter.add("text", function(text) {
  if(text == undefined){
      return text
  }
  return text + "s";
})

let colorSet = new am4core.ColorSet()

let range0 = axis.axisRanges.create();
range0.value = 0;
range0.endValue = 16;
range0.axisFill.fillOpacity = 1;
range0.axisFill.fill = colorSet.getIndex(0)
range0.axisFill.zIndex = - 1;

let range1 = axis.axisRanges.create();
range1.value = 16;
range1.endValue = 22;
range1.axisFill.fillOpacity = 1;
range1.axisFill.fill = colorSet.getIndex(2)
range1.axisFill.zIndex = -1;

let range2 = axis.axisRanges.create();
range2.value = 22;
range2.endValue = DEFAULT_OPTIONS_GAUGE_CHART.arcDelimiters[1];
range2.axisFill.fillOpacity = 1;
range2.axisFill.fill = colorSet.getIndex(4)
range2.axisFill.zIndex = -1;

let hand = chart.hands.push(new am4charts.ClockHand());
hand.innerRadius = am4core.percent(20);
    hand.startWidth = 4;
    // hand.pin.disabled = true;
    hand.value = 0;
// using chart.setTimeout method as the timeout will be disposed together with a chart
hand.showValue(valor, 1000, am4core.ease.cubicOut);


// Nuevo
let label = chart.radarContainer.createChild(am4core.Label);
    label.isMeasured = false;
    label.fontSize = 20;
    label.x = am4core.percent(20);
    label.y = am4core.percent(100);
    label.horizontalCenter = "middle";
    label.verticalCenter = "bottom";
    label.text = this.value + "s";

// chart.setTimeout(randomValue, 2000);

// function randomValue() {
//   hand.showValue(valor, 1000, am4core.ease.cubicOut);
//   chart.setTimeout(randomValue, 2000);
// }



/////fin

///original
// am4core.useTheme(am4themes_animated);
// am4core.addLicense("ch-custom-attribution");
// // Themes end

// console.log(' WATER CHART');
// am4core.useTheme(am4themes_animated);
// am4core.addLicense("ch-custom-attribution");

// let capacity = 6000;
// let value = 2500;
// let circleSize = 0.6;

// let component = am4core.create("chartdiv-gauge", am4core.Container)
// component.width = am4core.percent(100);
// component.height = am4core.percent(100);

// let chartContainer = component.createChild(am4core.Container);
// chartContainer.x = am4core.percent(50)
// chartContainer.y = am4core.percent(50)

// let circle = chartContainer.createChild(am4core.Circle);
// circle.fill = am4core.color("#dadada");

// let circleMask = chartContainer.createChild(am4core.Circle);

// let waves = chartContainer.createChild(am4core.WavedRectangle);
// waves.fill = am4core.color("#34a4eb");
// waves.mask = circleMask;
// waves.horizontalCenter = "middle";
// waves.waveHeight = 10;
// waves.waveLength = 30;
// waves.y = 500;
// circleMask.y = -500;

// component.events.on("maxsizechanged", function(){
//   let smallerSize = Math.min(component.pixelWidth, component.pixelHeight);  
//   let radius = smallerSize * circleSize / 2;

//   circle.radius = radius;
//   circleMask.radius = radius;
//   waves.height = smallerSize;
//   waves.width = Math.max(component.pixelWidth, component.pixelHeight);  

//   //capacityLabel.y = radius;

//   let labelRadius = radius + 20

//   capacityLabel.path = am4core.path.moveTo({x:-labelRadius, y:0}) + am4core.path.arcToPoint({x:labelRadius, y:0}, labelRadius, labelRadius);
//   capacityLabel.locationOnPath = 0.5;

//   setValue(value);
// })


// function setValue(value){
//   let y = - circle.radius - waves.waveHeight + (1 - value / capacity) * circle.pixelRadius * 2;
//   waves.animate([{property:"y", to:y}, {property:"waveHeight", to:10, from:15}, {property:"x", from:-50, to:0}], 5000, am4core.ease.elasticOut);
//   circleMask.animate([{property:"y", to:-y},{property:"x", from:50, to:0}], 5000, am4core.ease.elasticOut);
// }


// let label = chartContainer.createChild(am4core.Label)
// let formattedValue = component.numberFormatter.format(value, "#.#a");
// formattedValue = formattedValue.toUpperCase();

// label.text = formattedValue + "";
// label.fill = am4core.color("#fff");
// label.fontSize = 30;
// label.horizontalCenter = "middle";


// let capacityLabel = chartContainer.createChild(am4core.Label)

// let formattedCapacity = component.numberFormatter.format(capacity, "#.#a").toUpperCase();;

// capacityLabel.text = "Milisegundos";
// capacityLabel.fill = am4core.color("#34a4eb");
// capacityLabel.fontSize = 20;
// capacityLabel.textAlign = "middle";
// capacityLabel.padding(0,0,0,0);
// fn original



  }

}
