import { AfterViewInit, Component, HostListener, Input, OnInit, ViewChild } from '@angular/core';
import { PeityComponent } from '@xdf/graph';
import { PetiFormatter } from './formatter/formatter';

import * as am4core from "@amcharts/amcharts4/core";
import * as am4charts from "@amcharts/amcharts4/charts";
import am4themes_animated from "@amcharts/amcharts4/themes/animated";

declare var jQuery: any;

export class PetyWidgetModel {
  percent: number;
  up?: boolean;
  down?: boolean;
  value: number;
  history: Array<number>;
}

@Component({
  selector: 'byte-summary-peti-widget',
  templateUrl: './summary-peti-widget.component.html',
  styleUrls: ['./summary-peti-widget.component.scss']
})
export class SummaryPetiWidgetComponent implements OnInit, AfterViewInit {

  @Input() options = {
    width: 200,
    height: 60,
    fill: '#dfecf9',
    stroke: '#024990',
    strokeWidth: 1
  };
  @Input() values = [1, 12, 3, 15, 5, -1, 10];
  @Input() header;
  @Input() value: PetyWidgetModel;
  @Input() tooltip: string;
  @Input() formatter: PetiFormatter;

  @Input() areaColor;

  @Input() identificador: string;

  @ViewChild('peti', { static: false })
  peti: PeityComponent;

  constructor() { }

  ngOnInit() {
  }

  ngAfterViewInit() {
    var chart = am4core.create(this.identificador, am4charts.XYChart);
    var colors = new am4core.ColorSet();
    // chart.width = am4core.percent(100);
    // chart.height = 70;

    const dataForChart = [];

    const sorted_data = this.value.history.sort(function(a,b){
      return a[0] < b[0] ? -1:1;
    });
    let isDataDates = true;
    sorted_data.forEach(x => {
      if (typeof(x[0]) == 'string' ){
        const date = x[0];
        dataForChart.push({
          "date": date,
          "value": x[1]
        });
        isDataDates = false;
      }
      else{
        const date = new Date(x[0]);
        dataForChart.push({
          "date": date,
          "value": x[1]
        });
      }

      

    });
    chart.data = dataForChart;

    // chart.titles.template.fontSize = 10;
    // //chart.titles.template.textAlign = "left";
    // chart.titles.template.isMeasured = false;
    // chart.titles.create().text = "AAPL (Price)";

    chart.padding(30, 5, 2, 5);

    if (isDataDates){
      var dateAxis = chart.xAxes.push(new am4charts.DateAxis());
      // dateAxis.renderer.grid.template.disabled = true;
      // dateAxis.renderer.labels.template.disabled = true;
      dateAxis.startLocation = 0.5;
      dateAxis.endLocation = 0.7;
      dateAxis.cursorTooltipEnabled = false;
      // dateAxis.title.text = "Fecha";

      var valueAxis = chart.yAxes.push(new am4charts.ValueAxis());
      valueAxis.min = 0;
      // valueAxis.renderer.grid.template.disabled = true;
      // valueAxis.renderer.baseGrid.disabled = true;
      // valueAxis.renderer.labels.template.disabled = true;
      valueAxis.cursorTooltipEnabled = false;
      valueAxis.title.text = "Total";
      valueAxis.renderer.labels.template.adapter.add("text", function(text) {
        return text;
      });

      chart.cursor = new am4charts.XYCursor();
      // chart.cursor.lineY.disabled = true;
      chart.cursor.behavior = "none";

      var series = chart.series.push(new am4charts.LineSeries());
      series.tooltipText = "{date}: [bold]{value}";
      // series.tooltipHTML = "<img src='https://www.amcharts.com/lib/3/images/bicycle.png' style='vertical-align:bottom; margin-right: 10px; width:28px; height:21px;'><span style='font-size:14px; color:#000000;'><b>{date}: [bold]{value}</b></span>";
      series.dataFields.dateX = "date";
      series.dataFields.valueY = "value";
      series.fillOpacity = 0.6;
      series.tensionX = 0.8;
      series.strokeWidth = 2;
      series.stroke = colors.getIndex(this.areaColor);
      series.fill = series.stroke;

      // render data points as bullets
      var bullet = series.bullets.push(new am4charts.CircleBullet());
      bullet.circle.opacity = 0;
      bullet.circle.fill = colors.getIndex(this.areaColor);
      bullet.circle.propertyFields.opacity = "opacity";
      bullet.circle.radius = 3;
    }
    else{
      var categoryAxis1 = chart.xAxes.push(new am4charts.CategoryAxis());
      categoryAxis1.dataFields.category = "date";
      // categoryAxis1.renderer.labels.template.disabled = true;
      categoryAxis1.renderer.labels.template.adapter.add("text", function(text) {
        return '';
      });

      var valueAxis = chart.yAxes.push(new am4charts.ValueAxis());
      valueAxis.min = 0;
      // valueAxis.renderer.grid.template.disabled = true;
      // valueAxis.renderer.baseGrid.disabled = true;
      // valueAxis.renderer.labels.template.disabled = true;
      valueAxis.cursorTooltipEnabled = false;
      valueAxis.title.text = "Total";
      valueAxis.renderer.labels.template.adapter.add("text", function(text) {
        return text;
      });

      chart.cursor = new am4charts.XYCursor();
      // chart.cursor.lineY.disabled = true;
      chart.cursor.behavior = "none";

      var series = chart.series.push(new am4charts.LineSeries());
      series.tooltipText = "{date}: [bold]{value}";
      // series.tooltipHTML = "<img src='https://www.amcharts.com/lib/3/images/bicycle.png' style='vertical-align:bottom; margin-right: 10px; width:28px; height:21px;'><span style='font-size:14px; color:#000000;'><b>{date}: [bold]{value}</b></span>";
      series.dataFields.categoryX = "date";
      series.dataFields.valueY = "value";
      series.fillOpacity = 0.6;
      series.tensionX = 0.8;
      series.strokeWidth = 2;
      series.stroke = colors.getIndex(this.areaColor);
      series.fill = series.stroke;

      // render data points as bullets
      var bullet = series.bullets.push(new am4charts.CircleBullet());
      bullet.circle.opacity = 0;
      bullet.circle.fill = colors.getIndex(this.areaColor);
      bullet.circle.propertyFields.opacity = "opacity";
      bullet.circle.radius = 3;
    }

    

    
  }

  get formatterValue(): string|number {
    if (this.formatter) {
      return this.formatter.format(this.value.value);
    }
    return this.value.value;
  }

  @HostListener('window:resize', ['$event'])
  public onResize(event) {
    const width = event.target.innerWidth;
    this.updateWidth(width);
    
  }

  private updateWidth(width: number) {
    if (width >= 1600) {
      this.options.width = 300;
    } else if (width >= 1350) {
      this.options.width = 200;
    } else if (width >= 768) {
      this.options.width = 150;
    } else if (width >= 576) {
      this.options.width = 200;
    } else {
      this.options.width = 300;
    }
  }

}
